@import url("sphinxdoc.css");


div.admonition, div.warning {
    font-size: 1.00em;
    margin: 0em 0.em 0em 0;
    border: 0px;
    background-color: white;
}


div.admonition p.admonition-title,
div.warning p.admonition-title {
    padding: 0.1em 0 0.1em 0.0em;
    border: 0px solid #86989B; 
    color: black;
    border-bottom: 0px solid #86989B;
    background-color: white; 
}


body {
    min-width: 740px;
}

div.bodywrapper {
    margin: 0 300px 0 0;
}


div.sphinxsidebar {
    width: 275px;
}

dt:target, .highlight {
    background-color: #E3EFF1;
}

table.citation td.label {
  font-size: small;
  width: 120px;  /** will need to adjust based on largest label. **/
}

